#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef __BORLANDC__
#include <io.h>
#else
#include <unistd.h>
#endif
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include "fbb.h"

#ifndef O_BINARY
#define O_BINARY 0
#endif

int compare(const void *a, const void *b)
{
	return (strcmp(((info *) a)->indic.callsign, ((info *) b)->indic.callsign));
}

int main(int argc, char **argv)
{
	int fd, num_rec, i;
	long tot_calls = 0, tot_con = 0;
	struct stat sb;
	info *inf_data;

	if ((fd = open("inf.sys", O_RDONLY | O_BINARY)) < 0) {
		perror("Can't open inf.sys");
		return (1);
	}

	if (fstat(fd, &sb) < 0) {
		perror("Can't fstat() inf.sys");
		return (1);
	}

	if ((inf_data = malloc(sb.st_size)) == NULL) {
		perror("Can't allocate memory");
		return (1);
	}

	num_rec = sb.st_size / sizeof(info);

	read(fd, inf_data, sb.st_size);

	qsort(inf_data, num_rec, sizeof(info), compare);

	puts("Call   Home   ESBGMUP Cncts  Last con Name         QTH                  Locator\n"
		"===============================================================================");

	for (i = 0; i < num_rec; i++) {
		char *p, tmstr[9];
		struct tm *tm;

		if (inf_data[i].indic.callsign[0] == '\0')
			continue;
		if ((p = strchr(inf_data[i].home, '.')) != NULL)
			*p = '\0';
		tm = gmtime((time_t *) &inf_data[i].hcon);
		strftime(tmstr, sizeof(tmstr), "%Y%m%d", tm);
		printf("%-6s %-6s %s%s%s%s%s%s%s %-6ld %s %-12s %-20.20s %s\n",
			inf_data[i].indic.callsign, inf_data[i].home,
			(inf_data[i].flags & 1)    ? "E" : " ",
			(inf_data[i].flags & 8)    ? "S" : " ",
			(inf_data[i].flags & 16)   ? "B" : " ",
			(inf_data[i].flags & 64)   ? "G" : " ",
			(inf_data[i].flags & 128)  ? "M" : " ",
			(inf_data[i].flags & 512)  ? "U" : " ",
			(inf_data[i].flags & 2048) ? "P" : " ",
			inf_data[i].nbcon, tmstr, inf_data[i].prenom,
			inf_data[i].ville, inf_data[i].qra);
		tot_con += inf_data[i].nbcon;
		tot_calls++;
	}

	free(inf_data);

	printf("===============================================================================\n"
		"A total of %ld users made %ld connections.\n\n"
		"Flags:\n"
		"  E - Excluded\n"
		"  S - Sysop\n"
		"  B - BBS\n"
		"  G - Guest\n"
		"  M - Modem\n"
		"  U - Unproto allowed\n"
		"  P - PBBS\n", tot_calls, tot_con);

	close(fd);

	return (0);
}
